@@ -14,7 +14,7 @@
 # under the License.
 
 """ ``cord.client`` provides a simple Python client that allows you
-to query project resources through the Cord REST API.
+to query project resources through the Cord API.
 
 Here is a simple example for instantiating the client and obtaining project info:
 
@@ -25,21 +25,23 @@ Here is a simple example for instantiating the client and obtaining project info
     client = CordClient.initialize('YourProjectID', 'YourAPIKey')
     client.get_project()
 
-    :returns
-    Project info. See Project ORM for details.
+    Returns:
+        Project: A project record instance. See Project ORM for details.
 
 """
 
 import sys
 import logging
-import json
 import base64
+import uuid
 
 from cord.configs import CordConfig
 from cord.http.querier import Querier
 from cord.orm.project import Project
-from cord.orm.label_blurb import Label
+from cord.orm.label_row import LabelRow
 from cord.orm.model import Model, ModelInferenceParams
+from cord.orm.labeling_algorithm import LabelingAlgorithm, ObjectInterpolationParams
+from cord.utils.str_constants import *
 
 # Logging configuration
 logging.basicConfig(stream=sys.stdout,
@@ -52,7 +54,7 @@ logging.basicConfig(stream=sys.stdout,
 class CordClient(object):
     """
     Cord client. Allows you to query db items associated
-    with a project (e.g. labels, datasets).
+    with a project (e.g. label rows, datasets).
     """
 
     def __init__(self, querier, config):
@@ -105,15 +107,15 @@ class CordClient(object):
         """
         return self._querier.basic_getter(Project)
 
-    def get_label_blurb(self, uid):
+    def get_label_row(self, uid):
         """
-        Retrieves Label blurb
+        Retrieves label row
 
         Args:
             uid: A label_hash (uid) string.
 
         Returns:
-            Label: A label blurb instance.
+            LabelRow: A label row instance.
 
         Raises:
             AuthenticationError: If the project API key is invalid.
@@ -122,11 +124,11 @@ class CordClient(object):
             UnknownError: If an error occurs while retrieving the label.
             OperationNotAllowed: If the read operation is not allowed by the API key.
         """
-        return self._querier.basic_getter(Label, uid)
+        return self._querier.basic_getter(LabelRow, uid)
 
-    def save_label_blurb(self, uid, label):
+    def save_label_row(self, uid, label):
         """
-        Save existing Label blurb
+        Save existing label Row
 
         If you have a series of frame labels and have not updated answer
         dictionaries, call the construct_answer_dictionaries utils function
@@ -134,7 +136,7 @@ class CordClient(object):
 
         Args:
             uid: A label_hash (uid) string.
-            label: A label blurb instance.
+            label: A label row instance.
 
         Returns:
             Bool.
@@ -148,12 +150,13 @@ class CordClient(object):
             AnswerDictionaryError: If an object or classification instance is missing in answer dictionaries.
             CorruptedLabelError: If a blurb is corrupted (e.g. if the frame labels have more frames than the video).
         """
-        label = Label(label)
-        return self._querier.basic_setter(Label, uid, payload=label)
+        label = LabelRow(label)
+        return self._querier.basic_setter(LabelRow, uid, payload=label)
 
     def model_inference(self,
                         uid,
-                        file_path,
+                        file_paths=None,
+                        base64_strings=None,
                         conf_thresh=0.6,
                         iou_thresh=0.3,
                         device="cuda",
@@ -164,11 +167,12 @@ class CordClient(object):
 
         Args:
             uid: A model_iteration_hash (uid) string.
-            file_path: Local file path to image or video
+            file_paths: List of local file paths to image(s) or video(s) - if running inference on files
+            base64_strings: List of base 64 strings of image(s) or video(s) - if running inference on base64 strings
             conf_thresh: Confidence threshold (default 0.6)
             iou_thresh: Intersection over union threshold (default 0.3)
             device: Device (CPU or CUDA, default is CUDA)
-            detection_frame_range: Detection frame range (optional, if video)
+            detection_frame_range: Detection frame range (for videos)
 
         Returns:
             Inference results: A dict of inference results.
@@ -179,15 +183,34 @@ class CordClient(object):
             ResourceNotFoundError: If no model exists by the specified model_iteration_hash (uid).
             UnknownError: If an error occurs while running inference.
             FileTypeNotSupportedError: If the file type is not supported for inference (has to be an image or video)
-            MustSetDetectionRangeError: If a detection range is not set for video inference
+            DetectionRangeInvalidError: If a detection range is invalid for video inference
         """
+        if (file_paths is None and base64_strings is None) or (
+                file_paths is not None and len(file_paths) > 0 and base64_strings is not None and len(
+                base64_strings) > 0):
+            raise Exception("To run model inference, you must pass either a list of files or base64 strings")
+
         if detection_frame_range is None:
             detection_frame_range = []
 
-        file = open(file_path, 'rb').read()
+        files = []
+        if file_paths is not None:
+            for file_path in file_paths:
+                file = open(file_path, 'rb').read()
+                files.append({
+                    "uid": file_path,  # Add file path as inference identifier
+                    "base64_str": base64.b64encode(file).decode('utf-8')  # File to base64 string
+                })
+
+        elif base64_strings is not None:
+            for base64_string in base64_strings:
+                files.append({
+                    "uid": str(uuid.uuid4()),  # Add uuid as inference identifier
+                    "base64_str": base64_string.decode('utf-8')  # base64 string to utf-8
+                })
 
         params = ModelInferenceParams({
-            'file': base64.b64encode(file).decode('utf-8'),
+            'files': files,
             'conf_thresh': conf_thresh,
             'iou_thresh': iou_thresh,
             'device': device,
@@ -195,3 +218,70 @@ class CordClient(object):
         })
 
         return self._querier.basic_setter(Model, uid, payload=params)
+
+    def object_interpolation(self,
+                             key_frames,
+                             objects_to_interpolate,
+                             ):
+        """
+        Run object interpolation algorithm on project labels (requires an editor ontology and feature uid's).
+
+        Interpolation is supported for bounding box, polygon, and keypoint.
+
+        Args:
+            key_frames: Labels for frames to be interpolated. Key frames are consumed in the form:
+
+                "frame": {
+                    "objects": [
+                        {
+                            "objectHash": object_uid,
+                            "featureHash": feature_uid (from editor ontology),
+                            "polygon": {
+                                "0": {
+                                    "x": x1,
+                                    "y": y1,
+                                },
+                                "1": {
+                                    "x": x2,
+                                    "y": y2,
+                                },
+                                "2" {
+                                    "x": x3,
+                                    "y": y3,
+                                },
+                                ...,
+                            }
+                        },
+                        {
+                            ...
+                        }
+                    ]
+                },
+                "frame": {
+                    ...,
+                }
+
+            objects_to_interpolate: List of object uid's (hashes) of objects to interpolate
+
+        Returns:
+            Interpolation results: Full set of filled frames including interpolated objects
+
+        Raises:
+            AuthenticationError: If the project API key is invalid.
+            AuthorisationError: If access to the specified resource is restricted.
+            UnknownError: If an error occurs while running interpolation.
+        """
+        if len(key_frames) == 0 or len(objects_to_interpolate) == 0:
+            raise Exception("To run object interpolation, you must pass key frames and objects to interpolate")
+
+        interpolation_params = ObjectInterpolationParams({
+            'key_frames': key_frames,
+            'objects_to_interpolate': objects_to_interpolate,
+        })
+
+        params = LabelingAlgorithm({
+            'algorithm_name': INTERPOLATION,
+            'algorithm_parameters': interpolation_params,
+        })
+
+        return self._querier.basic_setter(LabelingAlgorithm, str(uuid.uuid4()), payload=params)
